function syst = SystemUpdate(k,syst,u,p_sys,p_sim)
% Perform update step discrete-time system.
% 
% Inputs:
%   k       current time step
%   syst    system struct, carrying current state, measurements etc.
%   u       input vector
%   p_sys   system parameter struct
%   p_sim   simulation parameters
%
% Outpus:
%   syst    system struct, updated.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [01.04.15, ST]    first version
% [02.05.15, ST]    removed input disturbance (only general disturbance)
% [02.08.15, ST]    some clean up


%% Noise
w = p_sys.w_lim.*(2*rand(p_sys.n_y,1)-1);           % measurement noise
vInp = p_sys.vInput_lim.*(2*rand(p_sys.n_u,1)-1);   % input process noise
v = p_sys.B * vInp;

if p_sim.flag_noNoise
    % turn off noise
    w = 0*w;
    v = 0*v;
end;


%% General disturbance
% General disturbance signal, defined in p_sys.
d = 0*v;
if p_sys.dist.enabled && p_sim.flag_generalDist
    % disturbance enabled
    d = p_sys.dist.E*p_sys.dist.d_signal(k,:)';     % E*d(k)
end;


%% System update and measurement
% State disturbance:
syst.d = v + d;

% State and measurement:
%   x(k) = Ax(k-1) + Bu(k-1) + Bminus*u(k-2) + v(k-1)
%   y(k) = Cx(k) + w(k)
syst.x = p_sys.A*syst.x + p_sys.B*u + syst.d;    
syst.y = p_sys.C*syst.x + w;
syst.u = u;     % save the current input
